<?php
/**
 * @file
 * Add system notifications as an option to several drush commands.
 */

/**
 * Implements hook_drush_help_alter().
 */
function drush_notify_drush_help_alter(&$command) {
  if ($command['command'] == 'global-options') {
    $command['options']['notify'] = 'Use system notifications to signal command completion.';
    $command['options']['notify-audio'] = 'Add an audio notification.';

    if ($command['#brief'] === FALSE) {
      $command['sub-options']['notify']['notify-cmd'] = 'Specify the shell command to trigger the notification.';
      $command['sub-options']['notify']['notify-cmd-audio'] = 'Specify the shell command to trigger the audio notification.';
    }
  }
}

/**
 * Implements hook_drush_help().
 */
function drush_notify_drush_help($section) {
  switch ($section) {
    case 'notify:cache-clear':
      return dt('Caches have been cleared.');
  }
}

/**
 * Implements hook_drush_exit().
 */
function drush_notify_drush_exit() {
  $cmd = drush_get_command();
  // pm-download is handled via richer data hook.
  if ($cmd['command'] != 'pm-download' && drush_notify_command_allowed($cmd['command'])) { 
    if (!$msg = drush_command_invoke_all('drush_help', 'notify:' . $cmd['command'])) {
      $msg = array(dt("Command '!command' completed.", array('!command' => $cmd['command'])));
    }

    drush_notify_send(implode("\n", $msg));
  }
}

/**
 * Implements hook_drush_pm_post_download().
 */
function drush_notify_drush_pm_post_download($project, $release) {
  // Use system notifications to signal download completion.
  // Note that this assumes libnotify-bin library installed (thus, Ubuntu-y)
  if (drush_notify_command_allowed('pm-download')) {
    if (!$msg = drush_command_invoke_all('drush_help', 'notify:' . $cmd['pm-download'])) {
      $msg = array(dt('Project !project (!version) downloaded to !install.', array('!project' => $project['name'], '!version' => $release['version'], '!install' => $project['project_install_location'])));
    }

    drush_notify_send(implode("\n", $msg));
  }
}

/**
 * Send a system notification.
 *
 * @param $msg
 *   Message to send via notification.
 */
function drush_notify_send($msg) {
  $icon = drush_normalize_path(DRUSH_BASE_PATH . '/drush_logo-black.png');
  $cmd = drush_get_option('notify-cmd', 'notify-send');
  drush_shell_exec("$cmd \"$msg\" -i $icon");

  $audio = drush_get_option('notify-audio', FALSE);
  if ($audio) {
    $cmd_audio = drush_get_option('notify-cmd-audio', 'spd-say');
    drush_shell_exec("$cmd_audio \"$msg\"");
  }
}

/**
 * Identify if the given Drush Command should trigger a notification.
 *
 * @param $command
 *   Name of the command.
 *
 * @return
 *   Boolean
 */
function drush_notify_command_allowed($command) {
  $notify = drush_get_option_list('notify');
  return $notify[0] == 1 || in_array($command, $notify);
}
